<?php

class DispletRetsIdxEmails{
	private function filter_message($message){
		return nl2br($message);
	}

	public function get_admin_email_address($displetretsidx_option){
		$admin_email_array = array();
		$admin_email = (!empty($displetretsidx_option['email'])) ? $displetretsidx_option['email'] : get_option('admin_email');
		if (strpos($admin_email, ',') !== false) {
			$admin_emails = explode(',', $admin_email);
			foreach ($admin_emails as $email) {
				$admin_email_array[] = trim($email);
			}
		}
		else{
			$admin_email_array[] = $admin_email;
		}
		return $admin_email_array;
	}

	public function get_assigned_emails($displetretsidx_option, $assigned_agent_id, $agent_only = false){
		if ($agent_only) {
			$assigned_email['admins_and_agent'] = array();
		}
		else{
			$assigned_email['admins_and_agent'] = self::get_admin_email_address($displetretsidx_option);
		}
		$duty_agent = self::get_duty_agent($assigned_agent_id);
		if (!empty($duty_agent) && !empty($duty_agent->user_email)) {
			$assigned_email['admins_and_agent'][] = $duty_agent->user_email;
			$assigned_email['address'] = $duty_agent->user_email;
			$assigned_email['name'] = (!empty($duty_agent->display_name)) ? $duty_agent->display_name : '';
		}
		else {
			$assigned_email['address'] = (!empty($displetretsidx_option['email_from_address'])) ? $displetretsidx_option['email_from_address'] : get_option('admin_email');
			$assigned_email['name'] = (!empty($displetretsidx_option['email_from_name'])) ? $displetretsidx_option['email_from_name'] : get_bloginfo('name');
		}
		return $assigned_email;
	}

	public function get_assigned_agent_id(){
		$current_user_id = get_current_user_id();
		if (!empty($current_user_id)) {
			$assigned_agent_id = get_user_meta($current_user_id, 'displet_agent_id', true);
			return $assigned_agent_id;
		}
	}

	public function get_duty_agent($agent_id){
		if (!empty($agent_id)) {
			$agent = get_userdata($agent_id);
			if (!empty($agent)) {
				return $agent;
			}
		}
	}

	private function get_headers($assigned_email){
		$headers = '';
		if (!empty($assigned_email['address'])) {
			$headers .= 'From: ';
			if (!empty($assigned_email['name'])) {
				$headers .= $assigned_email['name'] . ' ';
			}
			$headers .= '<' . $assigned_email['address'] . '>' . PHP_EOL;
		}
		$headers .= 'Content-type: text/html; charset=iso-8859-1' . PHP_EOL;
		return $headers;
	}

	public function get_signature($displetretsidx_option, $site_name, $site_url, $assigned_agent_id){
		if (!empty($assigned_agent_id)) {
			$email_signature = get_user_meta($assigned_agent_id, 'displet_email_signature', true);
		}
		else if (!empty($displetretsidx_option['email_signature'])) {
			$email_signature = $displetretsidx_option['email_signature'];
		}
		if (!empty($email_signature)) {
			$email_signature = str_replace('%%site_name%%', $site_name, stripslashes($email_signature));
			$email_signature = str_replace('%%site_url%%', $site_url, $email_signature);
			return $email_signature;
		}
	}

	private function replace_placeholders($template, $replacements){
		$template = stripslashes($template);
		if (!empty($replacements) && is_array($replacements)) {
			foreach ($replacements as $placeholder => $replacement) {
				$template = str_replace('%%' . $placeholder . '%%', $replacement, $template);
			}
		}
		return $template;
	}

	public function send_assigned_lead_message($lead_id, $assigned_agent_id){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$assigned_email = self::get_assigned_emails($displetretsidx_option, $assigned_agent_id, true);
		$headers = self::get_headers($assigned_email);
		$lead = get_userdata($lead_id);
		$email = $lead->user_email;
		$name = $lead->display_name;
		$phone = get_user_meta($lead_id, 'displet_phone', true);
		$site_name = get_bloginfo('name');
		$site_url = home_url();
		$leads_url = admin_url('admin.php?page=displet-lead-manager');
		$replacements = array(
			'user_name' => $name,
			'user_email' => $email,
			'user_phone' => $phone,
			'site_name' => $site_name,
			'site_url' => $site_url,
			'leads_url' => $leads_url
		);

		// Message to admin
		if (!empty($displetretsidx_option['email_template_assigned_lead'])) {
			$admin_message = self::replace_placeholders($displetretsidx_option['email_template_assigned_lead'], $replacements);
		}
		else {
			$admin_message =
				'A new lead has been assigned to you from ' . $site_url . '.' . PHP_EOL . PHP_EOL .
				'User Name: ' . $name . PHP_EOL .
				'User Email: ' . $email . PHP_EOL .
				'User Phone: ' . $phone . PHP_EOL . PHP_EOL .
				'To view all of your assigned leads, please visit ' . $leads_url;
		}
		$email_signature = self::get_signature($displetretsidx_option, $site_name, $site_url, $assigned_agent_id);
		if (!empty($email_signature)) {	
			$admin_message .= PHP_EOL . PHP_EOL . $email_signature;
		}
		if (!empty($displetretsidx_option['email_title_assigned_lead'])) {
			$admin_title = self::replace_placeholders($displetretsidx_option['email_title_assigned_lead'], $replacements);
		}
		else{
			$admin_title = 'A New Lead Has Been Assigned To You at ' . $site_url;
		}
		wp_mail($assigned_email['admins_and_agent'], $admin_title, self::filter_message($admin_message), $headers);
	}

	public function send_email_to_friend_request(){
		check_ajax_referer('displet_email_friend_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_email_friend_request'){
			if (!empty($_POST['email']) && is_email($_POST['email'])) {
				$displetretsidx_option = DispletRetsIdxSettings::get_option();
				$assigned_agent_id = self::get_assigned_agent_id();
				$assigned_email = self::get_assigned_emails($displetretsidx_option, $assigned_agent_id);
				$headers = self::get_headers($assigned_email);
				$site_name = get_bloginfo('name');
				$site_url = home_url();
				$current_user = wp_get_current_user();
				if (!empty($current_user->ID)) {
					$user = get_userdata($current_user->ID);
					if (!empty($user->display_name)) {
						$current_user->display_name = $user->display_name;
					}
				}
				$user_name = (!empty($current_user->display_name)) ? $current_user->display_name : 'A Friend';
				$user_email = (!empty($current_user->user_email)) ? $current_user->user_email : '';
				$address = trim($_POST['address']) . ', ' . trim($_POST['city']) . ', ' . trim($_POST['state']) . ' ' . trim($_POST['zip']);
				$replacements = array(
					'user_name' => $user_name,
					'user_email' => $user_email,
					'friend_name' => $_POST['name'],
					'friend_email' => $_POST['email'],
					'user_message' => $_POST['message'],
					'property_address' => $address,
					'property_url' => $_POST['url'],
					'site_name' => $site_name,
					'site_url' => $site_url
				);

				// Message to admin
				if (!empty($displetretsidx_option['email_template_email_friend_to_admin'])) {
					$admin_message = self::replace_placeholders($displetretsidx_option['email_template_email_friend_to_admin'], $replacements);
				}
				else {
					$admin_message =
						$_POST['url'] . PHP_EOL .
						'Property: ' . $address . PHP_EOL .
						'Friend Name: ' . $_POST['name'] . PHP_EOL .
						'Friend Email: ' . $_POST['email'] . PHP_EOL .
						'User Message: ' . $_POST['message'];
				}
				$email_signature = self::get_signature($displetretsidx_option, $site_name, $site_url, $assigned_agent_id);
				if (!empty($email_signature)) {	
					$admin_message .= PHP_EOL . PHP_EOL . $email_signature;
				}
				if (!empty($displetretsidx_option['email_title_email_friend_to_admin'])) {
					$admin_title = self::replace_placeholders($displetretsidx_option['email_title_email_friend_to_admin'], $replacements);
				}
				else {
					$admin_title = 'Property Emailed To A Friend from ' . $site_url;
				}
				wp_mail($assigned_email['admins_and_agent'], $admin_title, self::filter_message($admin_message), $headers);

				// Message to friend
				if (!empty($displetretsidx_option['email_template_email_friend_to_user'])) {
					$user_message = self::replace_placeholders($displetretsidx_option['email_template_email_friend_to_user'], $replacements);
				}
				else {
					$user_message =
						$_POST['url'] . PHP_EOL .
						'Property: ' . $_POST['address'] . ', ' . $_POST['city'] . ', ' . $_POST['state'] . ' ' . $_POST['zip'] . PHP_EOL .
						'Message from ' . $_POST['name'] . ': ' . $_POST['message'];
				}
				if (!empty($email_signature)) {	
					$user_message .= PHP_EOL . PHP_EOL . $email_signature;
				}
				if (!empty($displetretsidx_option['email_title_email_friend_to_user'])) {
					$user_title = self::replace_placeholders($displetretsidx_option['email_title_email_friend_to_user'], $replacements);
				}
				else {
					$user_title = $user_name . ' Thinks You\'ll Like ' . $address;
				}
				wp_mail($_POST['email'], $user_title, self::filter_message($user_message), $headers);
				
				echo 'Your message has been sent.';
			}
			else{
				echo 'Please enter a valid email address.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function send_new_user_registration($email, $name, $phone, $url, $username, $password, $assigned_agent_id){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$assigned_email = self::get_assigned_emails($displetretsidx_option, $assigned_agent_id);
		$headers = self::get_headers($assigned_email);
		$site_name = get_bloginfo('name');
		$site_url = home_url();
		$replacements = array(
			'user_name' => $name,
			'user_email' => $email,
			'user_phone' => $phone,
			'registration_url' => $url,
			'user_username' => $username,
			'site_name' => $site_name,
			'site_url' => $site_url
		);

		// Message to admin
		if (!empty($displetretsidx_option['email_template_new_registration_to_admin'])) {
			$admin_message = self::replace_placeholders($displetretsidx_option['email_template_new_registration_to_admin'], $replacements);
		}
		else {
			$admin_message =
				'User Name: ' . $name . PHP_EOL .
				'User Email: ' . $email . PHP_EOL .
				'User Phone: ' . $phone . PHP_EOL .
				'Registration Page: ' . $url;
		}
		$email_signature = self::get_signature($displetretsidx_option, $site_name, $site_url, $assigned_agent_id);
		if (!empty($email_signature)) {	
			$admin_message .= PHP_EOL . PHP_EOL . $email_signature;
		}
		if (!empty($displetretsidx_option['email_title_new_registration_to_admin'])) {
			$admin_title = self::replace_placeholders($displetretsidx_option['email_title_new_registration_to_admin'], $replacements);
		}
		else{
			$admin_title = 'A New User Has Registered at ' . $site_url;
		}
		wp_mail($assigned_email['admins_and_agent'], $admin_title, self::filter_message($admin_message), $headers);

		// Message to user
		if (!empty($displetretsidx_option['email_template_new_registration_to_user'])) {
			$replacements['user_password'] = $password;
			$user_message = self::replace_placeholders($displetretsidx_option['email_template_new_registration_to_user'], $replacements);
		}
		else{
			$user_message =
				'Thank you for registering at ' . home_url() . '.' . PHP_EOL .
				'Name: ' . $name . PHP_EOL .
				'Email: ' . $email . PHP_EOL .
				'Phone: ' . $phone . PHP_EOL .
				'Password: ' . $password;
		}
		if (!empty($email_signature)) {	
			$user_message .= PHP_EOL . PHP_EOL . $email_signature;
		}
		if (!empty($displetretsidx_option['email_title_new_registration_to_user'])) {
			$user_title = self::replace_placeholders($displetretsidx_option['email_title_new_registration_to_user'], $replacements);
		}
		else{
			$user_title = 'New Account Details for ' . $site_url;
		}
		wp_mail($email, $user_title, self::filter_message($user_message), $headers);
	}

	public function send_property_showing_request(){
		check_ajax_referer('displet_property_showing_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_property_showing_request'){
			if (!empty($_POST['name']) && !empty($_POST['email']) && !empty($_POST['phone'])) {
				if (is_email($_POST['email'])) {
					$displetretsidx_option = DispletRetsIdxSettings::get_option();
					$assigned_agent_id = self::get_assigned_agent_id();
					$assigned_email = self::get_assigned_emails($displetretsidx_option, $assigned_agent_id);
					$headers = self::get_headers($assigned_email);
					$site_name = get_bloginfo('name');
					$site_url = home_url();
					$address = trim($_POST['address']) . ', ' . trim($_POST['city']) . ', ' . trim($_POST['state']) . ' ' . trim($_POST['zip']);
					$replacements = array(
						'user_name' => $_POST['name'],
						'user_email' => $_POST['email'],
						'user_phone' => $_POST['phone'],
						'user_appointment' => $_POST['appointment1'],
						'user_appointment2' => $_POST['appointment2'],
						'user_message' => trim($_POST['message']),
						'property_address' => $address,
						'property_url' => $_POST['url'],
						'site_name' => $site_name,
						'site_url' => $site_url
					);

					// Send message to admin
					if (!empty($displetretsidx_option['email_template_property_showing_to_admin'])) {
						$admin_message = self::replace_placeholders($displetretsidx_option['email_template_property_showing_to_admin'], $replacements);
					}
					else {
						$admin_message = 
							$_POST['url'] . PHP_EOL .
							'Property: ' . $address . PHP_EOL .
							'Name: ' . $_POST['name'] . PHP_EOL .
							'Email: ' . $_POST['email'] . PHP_EOL .
							'Phone: ' . $_POST['phone'] . PHP_EOL .
							'Appointment 1: ' . $_POST['appointment1'] . PHP_EOL .
							'Appointment 2: ' . $_POST['appointment2'] . PHP_EOL .
							'Message: ' . $_POST['message'];
					}
					$email_signature = self::get_signature($displetretsidx_option, $site_name, $site_url, $assigned_agent_id);
					if (!empty($email_signature)) {	
						$admin_message .= PHP_EOL . PHP_EOL . $email_signature;
					}
					if (!empty($displetretsidx_option['email_title_property_showing_to_admin'])) {
						$admin_title = self::replace_placeholders($displetretsidx_option['email_title_property_showing_to_admin'], $replacements);
					}
					else {
						$admin_title = 'Property Showing Request';
					}
					wp_mail($assigned_email['admins_and_agent'], $admin_title, self::filter_message($admin_message), $headers);

					// Send message to user
					if (!empty($displetretsidx_option['email_template_property_showing_to_user'])) {
						$user_message = self::replace_placeholders($displetretsidx_option['email_template_property_showing_to_user'], $replacements);
					}
					else {
						$user_message = 
							$_POST['url'] . PHP_EOL .
							'Property: ' . $address . PHP_EOL .
							'Name: ' . $_POST['name'] . PHP_EOL .
							'Email: ' . $_POST['email'] . PHP_EOL .
							'Phone: ' . $_POST['phone'] . PHP_EOL .
							'Appointment 1: ' . $_POST['appointment1'] . PHP_EOL .
							'Appointment 2: ' . $_POST['appointment2'] . PHP_EOL .
							'Message: ' . $_POST['message'];
					}
					if (!empty($email_signature)) {	
						$user_message .= PHP_EOL . PHP_EOL . $email_signature;
					}
					if (!empty($displetretsidx_option['email_title_property_showing_to_user'])) {
						$user_title = self::replace_placeholders($displetretsidx_option['email_title_property_showing_to_user'], $replacements);
					}
					else {
						$user_title = 'Property Showing Request Received';
					}
					wp_mail($_POST['email'], $user_title, self::filter_message($user_message), $headers);

					// Send to Zapier
					$zap_user = array(
						'email' => $_POST['email'],
						'name' => $_POST['name'],
						'phone' => $_POST['phone'],
						'address' => $address,
						'first_appointment' => $_POST['appointment1'],
						'second_appointment' => $_POST['appointment2'],
						'message' => $_POST['message']
					);
					DispletRetsIdxUsers::send_to_zapier($zap_user, 'showing');
					
					echo 'Your request has been sent.';
				}
				else{
					echo 'Please enter a valid email address.';
				}
			}
			else{
				echo 'Please complete the required fields.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}
}

add_action('wp_ajax_displet_property_showing_request', array('DispletRetsIdxEmails', 'send_property_showing_request'));
add_action('wp_ajax_nopriv_displet_property_showing_request', array('DispletRetsIdxEmails', 'send_property_showing_request'));
add_action('wp_ajax_displet_email_friend_request', array('DispletRetsIdxEmails', 'send_email_to_friend_request'));
add_action('wp_ajax_nopriv_displet_email_friend_request', array('DispletRetsIdxEmails', 'send_email_to_friend_request'));

?>