<?php

class DispletRetsIdxUsers{
	public function register_user_ajax(){
		// Check nonce for security
		check_ajax_referer('displet_register_user_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_user_registration_request'){
			$displetretsidx_option = DispletRetsIdxSettings::get_option();
			if (!empty($displetretsidx_option['require_phone'])) {
				$phone = $_POST['phone'];
			}
			else{
				$phone = true;
			}
			if (!empty($_POST['name']) && !empty($_POST['email']) && !empty($phone)) {
				if (is_email($_POST['email'])) {
					$user_registration_response = self::register_user($_POST['email'], $_POST['name'], $_POST['phone'], $_POST['url'], $_POST['upstream_url'], $_POST['last_hash']);
					echo $user_registration_response;
				}
				else{
					echo 'Please enter a valid email address.';
				}
			}
			else{
				echo 'Please complete the required fields.';
			}
			die();
		}

		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function register_user($email, $name, $phone = '', $url = '', $upstream_url = '', $last_hash = '') {
		// Create WP User
		$random_password = wp_generate_password(12, false);
		$user_id = wp_create_user($email, $random_password, $email);
		if (is_wp_error($user_id)) {
			return $user_id->get_error_message();
		}
		$names = self::get_first_and_last_name($name);
		wp_update_user(array(
			'ID' => $user_id,
			'role' => 'displet_user',
			'display_name' => $names['first_name']
		));

		// Login user
		$user_info = get_userdata($user_id);
		$creds = array(
			'user_login' => $user_info->user_login,
			'user_password' => $random_password,
			'remember' => true
		);
		$user = wp_signon($creds, false);

		$cron_job_args = array(
			'user_id' => $user_id,
			'login' => $user_info->user_login,
			'password' => $random_password,
			'name' => $name,
			'first_name' => $names['first_name'],
			'last_name' => $names['last_name'],
			'email' => $email,
			'phone' => $phone,
			'url' => $url,
			'upstream_url' => $upstream_url,
			'last_hash' => $last_hash,
		);
		wp_schedule_single_event(time(), 'displetretsidx_new_user_cron_jobs', array($cron_job_args));

		return 'Successful Registration';
	}

	public function new_user_cron_jobs($args){
		extract($args);

		// Assign agent
		$assigned_agent_id = self::get_new_duty_agent_id();

		// Send email
		DispletRetsIdxEmails::send_new_user_registration($email, $name, $phone, $url, $login, $password, $assigned_agent_id);

		// Assign user meta
		update_user_meta($user_id, 'displet_lead', true);
		update_user_meta($user_id, 'displet_agent_id', $assigned_agent_id);
		update_user_meta($user_id, 'first_name', $first_name);
		update_user_meta($user_id, 'last_name', $last_name);
		update_user_meta($user_id, 'nickname', $name);
		update_user_meta($user_id, 'displet_phone', $phone);
		update_user_meta($user_id, 'displet_registration_url', $url);
		update_user_meta($user_id, 'displet_registration_upstream_url', $upstream_url);
		update_user_meta($user_id, 'displet_last_hash', $last_hash);

		// Send to Zapier
		$zap_user = array(
			'email' => $email,
			'name' => $name,
			'phone' => $phone,
			'registered_at' => $url,
			'came_from' => $upstream_url
		);
		self::send_to_zapier($zap_user, 'registration');

		// Send to Displet API
		$api_user_id = self::create_api_user($email, $first_name, $last_name);
		update_user_meta($user_id, 'displet_api_user_id', $api_user_id);
	}

	public function login_user_ajax() {
		// Check nonce for security
		check_ajax_referer('displet_login_user_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displetretsidx_user_signon'){
			if (!empty($_POST['email']) && !empty($_POST['password'])) {
				$creds = array();
				$creds['user_login'] = $_POST['email'];
				$creds['user_password'] = $_POST['password'];
				$creds['remember'] = true;
				$user = wp_signon($creds, false);
				if (is_wp_error($user)) {
					echo $user->get_error_message();
				}
				else{
					echo 'Successful Login';
				}
			}
			else {
				echo 'Please complete the required fields.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function check_user_ajax() {
		// Check nonce for security
		check_ajax_referer('displet_check_user_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_user_check_request'){
			if (!empty($_POST['email'])) {
				$user_exists = self::check_user($_POST['email']);
				if ($user_exists) {
					echo 'User Exists and Logged In';
				}
			}
			else {
				echo 'There is no email account associated with this user.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function check_user($email){
		$user_id = username_exists($email);
		if (!empty($user_id)) {
			wp_set_auth_cookie($user_id, true);
			return true;
		}
		else{
			return false;
		}
	}

	public function check_login_ajax(){
		// Check nonce for security
		check_ajax_referer('displet_check_login_nonce');

		if (is_user_logged_in()) {
			echo 'Successful Login';
		}
		else{
			echo 'User Not Logged In';
		}
		die();
	}

	public function get_new_duty_agent_id(){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$duty_agent_array_position = get_option('displet_rets_idx_duty_agent_array_position');
		if (!empty($displetretsidx_option['duty_agent'])) {
			$duty_agents = array();
			foreach ($displetretsidx_option['duty_agent'] as $agent_id => $value) {
				if ($value == 'true'){
					$duty_agents[] = $agent_id;
				}
			}
			if (!empty($duty_agents)) {
				if (!empty($duty_agent_array_position) && intval($duty_agent_array_position) < count($duty_agents)) {
					$i = intval($duty_agent_array_position);
				}
				else{
					$i = 0;
				}
				$duty_agent_array_position = $i;
				$duty_agent_array_position++;
				update_option('displet_rets_idx_duty_agent_array_position', $duty_agent_array_position);
				return $duty_agents[$i];
			}
		}
	}

	public function create_api_user($email, $first_name, $last_name){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$query_url = 'http://api.displet.com/users';
		$query_args = array(
			'timeout' => 10,
			'headers' => array(
				'referer' => home_url()
			),
			'body' => array(
				'user' => array(
					'email' => $email,
					'first_name' => $first_name,
					'last_name' => $last_name
				),
				'authentication_token' => $displetretsidx_option['displet_app_key']
			)
		);
		$json_array = wp_remote_post($query_url, $query_args);
		if (is_array($json_array)) {
			$json_data = json_decode($json_array['body']);
			if (!empty($json_data->id)) {
				return $json_data->id;
			}
		}
		return false;
	}

	public function update_api_user($api_user_id, $user_array){
		if (!empty($user_array)) {
			$displetretsidx_option = DispletRetsIdxSettings::get_option();
			$query_url = 'http://api.displet.com/users/' . $api_user_id;
			$query_args = array(
				'method' => 'PUT',
				'timeout' => 10,
				'headers' => array(
					'referer' => home_url()
				),
				'body' => array(
					'user' => $user_array,
					'authentication_token' => $displetretsidx_option['displet_app_key']
				)
			);
			$json_array = wp_remote_post($query_url, $query_args);
		}
	}

	public function get_api_user($user_id){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$query_url = 'http://api.displet.com/users/' . $user_id;
		$query_args = array(
			'timeout' => 10,
			'headers' => array(
				'referer' => home_url()
			),
			'body' => array(
				'authentication_token' => $displetretsidx_option['displet_app_key']
			)
		);
		$json_array = wp_remote_get($query_url, $query_args);
		if (is_array($json_array)) {
			$json_data = json_decode($json_array['body']);
			return $json_data;
		}
	}

	public function get_api_saved_searches($user_id){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$query_url = 'http://api.displet.com/users/' . $user_id . '/saved_searches';
		$query_args = array(
			'timeout' => 10,
			'headers' => array(
				'referer' => home_url()
			),
			'body' => array(
				'authentication_token' => $displetretsidx_option['displet_app_key']
			)
		);
		$json_array = wp_remote_get($query_url, $query_args);
		if (is_array($json_array)) {
			$json_data = json_decode($json_array['body']);
			return $json_data;
		}
	}

	public function get_api_saved_search($user_id, $search_id){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$query_url = 'http://api.displet.com/users/' . $user_id . '/saved_searches/' . $search_id;
		$query_args = array(
			'timeout' => 10,
			'headers' => array(
				'referer' => home_url()
			),
			'body' => array(
				'authentication_token' => $displetretsidx_option['displet_app_key']
			)
		);
		$json_array = wp_remote_get($query_url, $query_args);
		if (is_array($json_array)) {
			$json_data = json_decode($json_array['body']);
			return $json_data;
		}
	}

	public function create_api_saved_search($user_id, $search_name, $search_criteria){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$query_url = 'http://api.displet.com/users/' . $user_id . '/saved_searches';
		$query_args = array(
			'timeout' => 10,
			'headers' => array(
				'referer' => home_url()
			),
			'body' => array(
				//'saved_search' => array(
					'name' => $search_name,
					'search_criteria' => $search_criteria,
					'email_frequency' => 1,
				//),
				'authentication_token' => $displetretsidx_option['displet_app_key']
			)
		);
		$json_array = wp_remote_post($query_url, $query_args);
		if (is_array($json_array)) {
			$json_data = json_decode($json_array['body']);
			if (!empty($json_data->id)) {
				return $json_data->id;
			}
		}
		return false;
	}

	public function update_api_saved_search($user_id, $search_id, $search_criteria){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$query_url = 'http://api.displet.com/users/' . $user_id . '/saved_searches/' . $search_id;
		$query_args = array(
			'method' => 'PUT',
			'timeout' => 10,
			'headers' => array(
				'referer' => home_url()
			),
			'body' => array(
				//'saved_search' => array(
					'search_criteria' => $search_criteria,
				//),
				'authentication_token' => $displetretsidx_option['displet_app_key']
			)
		);
		$json_array = wp_remote_post($query_url, $query_args);
		if (is_array($json_array)) {
			$json_data = json_decode($json_array['body']);
			return $json_data;
		}
	}

	public function delete_api_saved_search($user_id, $search_id){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$query_url = 'http://api.displet.com/users/' . $user_id . '/saved_searches/' . $search_id;
		$query_args = array(
			'method' => 'DELETE',
			'timeout' => 10,
			'headers' => array(
				'referer' => home_url()
			),
			'body' => array(
				'authentication_token' => $displetretsidx_option['displet_app_key']
			)
		);
		$json_array = wp_remote_post($query_url, $query_args);
		if (is_array($json_array)) {
			$json_data = json_decode($json_array['body']);
			return $json_data;
		}
	}

	public function update_last_login($user_login, $user){
		update_user_meta($user->ID, 'displet_last_login', date('Y/m/d \a\t g:ia', current_time('timestamp')));
	}

	public function update_viewed_searches(){
		// Check nonce for security
		check_ajax_referer('displet_update_searches_nonce');
		
		if (isset($_POST['action']) && $_POST['action'] == 'displet_update_searches' && !empty($_POST['last_hash'])){
			$user_id = get_current_user_id();
			if ($user_id != 0 && current_user_can('displet_save_searches') && !current_user_can('manage_options')) {
				$user_searches = get_user_meta($user_id, 'displet_user_hashes', true);
				$user_searches[] = $_POST['last_hash'];
				update_user_meta($user_id, 'displet_user_hashes', $user_searches);
			}
		}
		die();
	}

	public function update_viewed_properties($url = '', $address = '', $price = '', $zip = '', $sq_ft = ''){
		$user_id = get_current_user_id();
		if ($user_id != 0 && current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
			$user_searches = get_user_meta($user_id, 'displet_user_properties', true);
			$user_searches[] = array(
				'url' => $url,
				'address' => $address,
				'price' => $price,
				'zip' => $zip,
				'sq_ft' => $sq_ft
			);
			update_user_meta($user_id, 'displet_user_properties', $user_searches);
		}
	}

	public function delete_users(){
		// Check nonce for security
		check_ajax_referer('displet_delete_users_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_delete_users_request'){
			if (!empty($_POST['displet_users'])) {
				$error_occurred = false;
				foreach ($_POST['displet_users'] as $user){
					$deleted[] = wp_delete_user($user);
				}
				foreach ($deleted as $was_deleted) {
					if (!$was_deleted){
						$error_occurred = true;
					}
				}
				if ($error_occurred){
					echo 'There was an error processing your request.';
				}
				else{
					echo 'Succesful Deletion';
				}
			}
			die();
		}
	}

	public function reassign_users(){
		// Check nonce for security
		check_ajax_referer('displet_reassign_users_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_reassign_users_request'){
			if (!empty($_POST['displet_users'])) {
				if (!empty($_POST['displet_agent'])) {
					foreach ($_POST['displet_users'] as $user){
						update_user_meta($user, 'displet_agent_id', $_POST['displet_agent']);
						DispletRetsIdxEmails::send_assigned_lead_message($user, $_POST['displet_agent']);
					}
					echo 'Succesful Assignation';
				}
				else{
					echo 'No agent has been selected.';
				}
			}
			die();
		}
	}

	public function save_property(){
		// Check nonce for security
		check_ajax_referer('displet_save_property_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_save_property_request'){
			if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
				if (!empty($_POST['type'])) {
					$displetretsidx_option = DispletRetsIdxSettings::get_option();
					$user_id = get_current_user_id();
					if ($user_id != 0) {
						$base = get_permalink($displetretsidx_option['property_details_page_id']);
						$url = str_replace($base, '', $_POST['url']);
						$saved_properties = get_user_meta($user_id, 'displet_saved_properties', true);
						$saved_properties[] = array(
							'sysid' => $_POST['sysid'],
							'url' => $url,
							'type' => $_POST['type'],
							'rating' => $_POST['rating'],
							'message' => $_POST['message'],
						);
						update_user_meta($user_id, 'displet_saved_properties', $saved_properties);
						echo 'This property has been saved.';
					}
					else{
						echo 'You are not logged in. Please sign in or create an account and try again.';
					}
				}
				else{
					echo 'Please select Save as favorite, Save as possibility, or Just save notes.';
				}
			}
			else{
				echo 'The current user does not have the capability to save properties.';
			}
			die();
		}
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function delete_saved_properties(){
		// Check nonce for security
		check_ajax_referer('displet_delete_properties_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_delete_properties_request'){
			if (!empty($_POST['displet_properties'])) {
				$user_id = get_current_user_id();
				$saved_properties = get_user_meta($user_id, 'displet_saved_properties');
				$delete_properties = $_POST['displet_properties'];
				foreach ($delete_properties as $property_id){
					unset($saved_properties[0][$property_id]);
				}
				$saved_properties[0] = array_values($saved_properties[0]);
				update_user_meta($user_id, 'displet_saved_properties', $saved_properties[0]);
				echo 'Succesful Deletion';
			}
			else{
				echo 'No property was selected for deletion.';
			}
			die();
		}
		else {
			echo 'There was an error processing your request. Please try again.';
		}
		die();
	}

	public function save_search(){
		// Check nonce for security
		check_ajax_referer('displet_save_search_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_save_search_request'){
			if (!empty($_POST['hash'])) {
				$user_id = get_current_user_id();
				if (!empty($user_id)) {
					if (current_user_can('displet_save_searches') && !current_user_can('manage_options')) {
						if (!empty($_POST['search_name'])) {
							$saved_searches = get_user_meta($user_id, 'displet_saved_searches', true);
							$name_exists = false;
							if (!empty($saved_searches) && is_array($saved_searches)) {
								foreach ($saved_searches as $saved_search) {
									if ($saved_search['name'] == $_POST['search_name']){
										$name_exists = true;
									}
								}
							}
							if (empty($name_exists)) {
								$this_search = array(
									'hash' => $_POST['hash'],
									'name' => $_POST['search_name']
								);
								$api_user_id = get_user_meta($user_id, 'displet_api_user_id', true);
								if (!empty($api_user_id)) {
									$saved_search_criteria = self::get_search_criteria_from_hash($_POST['hash']);
									$saved_search_id = self::create_api_saved_search($api_user_id, $_POST['search_name'], $saved_search_criteria);
									if (isset($saved_search_id)) {
										$this_search['api_id'] = $saved_search_id;
									}
								}
								$saved_searches[] = $this_search;
								update_user_meta($user_id, 'displet_saved_searches', $saved_searches);
								echo 'Saved Search';
								die();
							}
							else {
								echo 'The search "' . $_POST['search_name'] . '" already exists. Please select a different name.';
							}
						}
						else {
							echo 'Please name the search.';
						}
					}
					else{
						echo 'The current user does not have the capability to save properties.';
					}
				}
				else{
					echo 'Not Logged In';
				}
			}
			else{
				echo 'Please perform a search to save.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function delete_saved_searches(){
		// Check nonce for security
		check_ajax_referer('displet_delete_searches_nonce');

		if (isset($_POST['action']) && $_POST['action'] == 'displet_delete_searches_request'){
			if (!empty($_POST['displet_searches'])) {
				$user_id = get_current_user_id();
				$saved_searches = get_user_meta($user_id, 'displet_saved_searches', true);
				$delete_searches = $_POST['displet_searches'];
				foreach ($delete_searches as $search){
					unset($saved_searches[$search['array_index']]);
					if (!empty($search['api_id'])) {
						$api_user_id = get_user_meta($user_id, 'displet_api_user_id', true);
						self::delete_api_saved_search($api_user_id, $search['api_id']);
					}
				}
				$saved_searches = array_values($saved_searches);
				update_user_meta($user_id, 'displet_saved_searches', $saved_searches);
				echo 'Succesful Deletion';
			}
			else{
				echo 'No property was selected for deletion.';
			}
			die();
		}
		else {
			echo 'There was an error processing your request. Please try again.';
		}
		die();
	}

	public function set_referral_cookie(){
		if (empty($_COOKIE['displetretsidx_referring_site'])) {
        	setcookie('displetretsidx_referring_site', $_SERVER['HTTP_REFERER'], 0, '/');
		}
	}

	private function get_search_criteria_from_hash($hash){
		$trimmed_saved_search_criteria = rtrim(ltrim($hash, '#'), '/');
		$saved_search_criteria = str_replace('/', '&', $trimmed_saved_search_criteria);
		return $saved_search_criteria;
	}

	private function get_first_and_last_name($name){
		$names = array();
		preg_match('/[^\s]+/', $name, $first_name_match);
		if (!empty($first_name_match)) {
			$names['first_name'] = $first_name_match[0];
		}
		else{
			$names['first_name'] = $name;
		}
		$names['last_name'] = trim(str_replace($names['first_name'], '', $name));
		return $names;
	}

	private function upgrade_users_to_role_2(){
		$leads = get_users(array(
			'meta_key' => 'displet_lead',
			'meta_value' => true
		));
		if (!empty($leads)) {
			foreach ($leads as $lead) {
				$name = get_user_meta($lead->ID, 'nickname', true);
				$names = self::get_first_and_last_name($name);
				if (!empty($names)) {		
					wp_update_user(array(
						'ID' => $lead->ID,
						'display_name' => $names['first_name']
					));
					update_user_meta($lead->ID, 'first_name', $names['first_name']);
					update_user_meta($lead->ID, 'last_name', $names['last_name']);
				}
				$api_user_id = get_user_meta($lead->ID, 'displet_api_user_id', true);
				if (empty($api_user_id)) {
					$api_user_id = self::create_api_user($lead->user_email, $names['first_name'], $names['last_name']);
					update_user_meta($lead->ID, 'displet_api_user_id', $api_user_id);
				}
				$saved_searches = get_user_meta($lead->ID, 'displet_saved_searches', true);
				if (!empty($saved_searches) && !empty($api_user_id)) {
					foreach ($saved_searches as $saved_search) {
						$search_name = wp_generate_password(12, false);
						$search_criteria = self::get_search_criteria_from_hash($saved_search);
						self::create_api_saved_search($api_user_id, $search_name, $search_criteria);
					}
				}
			}
		}
	}

	public function add_custom_user_role_for_leads(){
		// Making changes in this function requires increasing the user_role_version
		$user_role_version = get_option('displet_rets_idx_user_role_version');
		if (empty($user_role_version) || $user_role_version < 2){
			remove_role('displet_user');
			$capabilities = array(
				'read' => true,
				'displet_save_properties' => true,
				'displet_save_searches' => true
			);
			add_role('displet_user', 'Real Estate Search User', $capabilities);
			if ($user_role_version < 2) {
				self::upgrade_users_to_role_2();
			}
			update_option('displet_rets_idx_user_role_version', 2);
		}
	}

	public function add_custom_user_role_for_agents(){
		// Making changes in this function requires increasing the user_role_version_admin
		$user_role_version_agents = get_option('displet_rets_idx_user_role_version_agents');
		if (empty($user_role_version_agents) || $user_role_version_agents < 2){
			remove_role('displet_agent');
			$capabilities = array(
				'read' => true,
				'displet_save_properties' => true,
				'displet_save_searches' => true,
				'displet_view_leads' => true,
				'edit_users' => true
			);
			add_role('displet_agent', 'Displet RE Agent', $capabilities);
			update_option('displet_rets_idx_user_role_version_agents', 2);
		}
	}

	public function add_custom_capabilities_to_admin(){
		// Making changes in this function requires increasing the user_role_version_agents
		$user_role_version_admin = get_option('displet_rets_idx_user_role_version_admin');
		if (empty($user_role_version_admin) || $user_role_version_admin < 1){
			$role = get_role('administrator');
			$role->add_cap('displet_view_leads');
			update_option('displet_rets_idx_user_role_version_admin', 1);
		}
	}

	public function add_saved_properties_menu(){
		if (!current_user_can('manage_options')) {
			add_menu_page('Saved Properties', 'Saved Properties', 'displet_save_properties', 'displetretsidx_saved_properties', array('DispletRetsIdxUsers', 'add_saved_properties_page'), plugins_url('displet-retsidx-beta/css/images/saved-properties-icon.png'), 71);
		}
	}

	public function add_saved_properties_page(){
		require_once dirname(__FILE__) . '/users/displet-saved-properties-page.php';
	}

	public function add_saved_searches_menu(){
		if (!current_user_can('manage_options')) {
			add_menu_page('Saved Searches', 'Saved Searches', 'displet_save_searches', 'displetretsidx_saved_searches', array('DispletRetsIdxUsers', 'add_saved_searches_page'), plugins_url('displet-retsidx-beta/css/images/saved-searches-icon.png'), 72);
		}
	}

	public function add_saved_searches_page(){
		require_once dirname(__FILE__) . '/users/displet-saved-searches-page.php';
	}

	public function add_search_properties_to_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
			$displetretsidx_option = DispletRetsIdxSettings::get_option();
			$href = get_permalink($displetretsidx_option['search_results_page_id']);
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_search_properties_toolbar',
    			'title' => 'Search Properties',
    			'href' => $href
    		));
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_search_properties_toolbar_SUB',
    			'title' => 'Search Properties',
    			'href' => $href,
    			'parent' => 'site-name'
    		));
		}
	}

	public function add_saved_properties_to_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_properties_toolbar',
    			'title' => 'Saved Properties',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_properties')
    		));
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_properties_toolbar_sub',
    			'title' => 'Saved Properties',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_properties'),
    			'parent' => 'site-name'
    		));
		}
	}

	public function add_saved_searches_to_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_searches') && !current_user_can('manage_options')) {
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_searches_toolbar',
    			'title' => 'Saved Searches',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_searches')
    		));
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_searches_toolbar_sub',
    			'title' => 'Saved Searches',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_searches'),
    			'parent' => 'site-name'
    		));
		}
	}

	public function clean_up_user_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
    		$wp_admin_bar->remove_node('wp-logo');
    		$wp_admin_bar->remove_node('search');
		}
	}

	public function clean_up_user_dashboard(){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
			remove_meta_box('dashboard_primary', 'dashboard', 'side');
			remove_meta_box('dashboard_secondary', 'dashboard', 'side');
		}
	}

	public function send_to_zapier($user_data, $zap){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		if (!empty($displetretsidx_option['zapier_' . $zap . '_url'])) {
			$request = array(
				'user-agent' => 'Displet',
				'body' => $user_data,
				'referer' => home_url()
			);
			$urls = explode("\n", $displetretsidx_option['zapier_' . $zap . '_url']);
			foreach ($urls as $url) {
				$post_url = trim($url);
				if (!empty($post_url)) {
					wp_remote_post($post_url, $request);
				}
			}
		}
	}

	public function add_to_agent_page($user){
		if (user_can($user->ID, 'displet_view_leads')) {

			?>
				<h3>Displet Settings</h3>
				<table class="form-table">
					<tr>
						<th><label for="displet_email_signature">Email Signature</label></th>
						<td>
							<textarea name="displet_email_signature" id="displet_email_signature" rows="5" cols="30"><?php echo esc_attr(get_the_author_meta('displet_email_signature', $user->ID)); ?></textarea><br />
							<span class="description">Displayed at the bottom of each outbound email.</span>
						</td>
					</tr>
				</table>
			<?php
		}
	}

	public function save_from_agent_page($user_id){
		$saved = false;
		if (user_can($user_id, 'displet_view_leads') && current_user_can('edit_user', $user_id)) {
			update_user_meta($user_id, 'displet_email_signature', $_POST['displet_email_signature']);
			$saved = true;
		}
		return true;
	}

	public function save_from_user_page($user_id){
		if (user_can($user_id, 'displet_save_properties') && current_user_can('edit_user', $user_id)) {
			$api_user_id = get_user_meta($user_id, 'displet_api_user_id', true);
			if (!empty($api_user_id)) {
				$user_array = array();
				if (!empty($_POST['first_name'])) {
					$user_array['first_name'] = $_POST['first_name'];
				}
				if (!empty($_POST['last_name'])) {
					$user_array['last_name'] = $_POST['last_name'];
				}
				if (!empty($_POST['email'])) {
					$user_array['email'] = $_POST['email'];
				}
				self::update_api_user($api_user_id, $user_array);
			}
		}
	}
}

function echo_stuff(){
	//var_dump(DispletRetsIdxUsers::create_api_user('asjdkfj@skldjf.com', 'Josh', 'Davis'));
	//DispletRetsIdxUsers::update_api_user(3096, array('first_name' => 'asdfasdf'));
	//var_dump(DispletRetsIdxUsers::get_api_user(3096));
	//var_dump(get_user_meta(138, 'displet_here', true));
	//var_dump(DispletRetsIdxUsers::create_api_saved_search(3096, 'jaksdfi348uklsasdfasdfdjfu8rjlsf', 'zip=78732'));
	//var_dump(DispletRetsIdxUsers::get_api_saved_searches(37849));
	//var_dump(DispletRetsIdxUsers::get_api_saved_search(3096, 24));
	//var_dump(DispletRetsIdxUsers::update_api_saved_search(3096, 19, array('zip' => '78702', 'max_list_price' => '2000')));
	//var_dump(DispletRetsIdxUsers::delete_api_saved_search(3096, 34));
	//var_dump(DispletRetsIdxUsers::upgrade_users_to_role_2());
}

//add_action('pre_get_posts', 'echo_stuff');

add_action('wp_ajax_displet_user_registration_request', array('DispletRetsIdxUsers', 'register_user_ajax'));
add_action('wp_ajax_nopriv_displet_user_registration_request', array('DispletRetsIdxUsers', 'register_user_ajax'));
add_action('wp_ajax_displetretsidx_user_signon', array('DispletRetsIdxUsers', 'login_user_ajax'));
add_action('wp_ajax_nopriv_displetretsidx_user_signon', array('DispletRetsIdxUsers', 'login_user_ajax'));
add_action('wp_ajax_displet_user_check_request', array('DispletRetsIdxUsers', 'check_user_ajax'));
add_action('wp_ajax_nopriv_displet_user_check_request', array('DispletRetsIdxUsers', 'check_user_ajax'));
add_action('wp_ajax_displet_check_login_request', array('DispletRetsIdxUsers', 'check_login_ajax'));
add_action('wp_ajax_nopriv_displet_check_login_request', array('DispletRetsIdxUsers', 'check_login_ajax'));
add_action('wp_ajax_displet_delete_users_request', array('DispletRetsIdxUsers', 'delete_users'));
add_action('wp_ajax_displet_reassign_users_request', array('DispletRetsIdxUsers', 'reassign_users'));
add_action('wp_login', array('DispletRetsIdxUsers', 'update_last_login'), 10, 2);
add_action('wp_ajax_displet_update_searches', array('DispletRetsIdxUsers', 'update_viewed_searches'));
add_action('wp_ajax_nopriv_displet_update_searches', array('DispletRetsIdxUsers', 'update_viewed_searches'));
add_action('wp_ajax_displet_update_properties', array('DispletRetsIdxUsers', 'update_viewed_properties'));
add_action('wp_ajax_nopriv_displet_update_properties', array('DispletRetsIdxUsers', 'update_viewed_properties'));
add_action('wp_ajax_displet_save_property_request', array('DispletRetsIdxUsers', 'save_property'));
add_action('wp_ajax_nopriv_displet_save_property_request', array('DispletRetsIdxUsers', 'save_property'));
add_action('wp_ajax_displet_delete_properties_request', array('DispletRetsIdxUsers', 'delete_saved_properties'));
add_action('wp_ajax_nopriv_displet_delete_properties_request', array('DispletRetsIdxUsers', 'delete_saved_properties'));
add_action('wp_ajax_displet_save_search_request', array('DispletRetsIdxUsers', 'save_search'));
add_action('wp_ajax_nopriv_displet_save_search_request', array('DispletRetsIdxUsers', 'save_search'));
add_action('wp_ajax_displet_delete_searches_request', array('DispletRetsIdxUsers', 'delete_saved_searches'));
add_action('wp_ajax_nopriv_displet_delete_searches_request', array('DispletRetsIdxUsers', 'delete_saved_searches'));
add_action('init', array('DispletRetsIdxUsers', 'set_referral_cookie'));
add_action('init', array('DispletRetsIdxUsers', 'add_custom_user_role_for_leads'));
add_action('init', array('DispletRetsIdxUsers', 'add_custom_user_role_for_agents'));
add_action('init', array('DispletRetsIdxUsers', 'add_custom_capabilities_to_admin'));
add_action('admin_menu', array('DispletRetsIdxUsers', 'add_saved_properties_menu'));
add_action('admin_menu', array('DispletRetsIdxUsers', 'add_saved_searches_menu'));
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'add_search_properties_to_toolbar'), 999);
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'add_saved_properties_to_toolbar'), 999);
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'add_saved_searches_to_toolbar'), 999);
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'clean_up_user_toolbar'), 9999);
add_action('admin_menu', array('DispletRetsIdxUsers', 'clean_up_user_dashboard'), 9999);
add_action('show_user_profile', array('DispletRetsIdxUsers', 'add_to_agent_page'));
add_action('edit_user_profile', array('DispletRetsIdxUsers', 'add_to_agent_page'));
add_action('personal_options_update', array('DispletRetsIdxUsers', 'save_from_agent_page'));
add_action('edit_user_profile_update', array('DispletRetsIdxUsers', 'save_from_agent_page'));
add_action('personal_options_update', array('DispletRetsIdxUsers', 'save_from_user_page'));
add_action('edit_user_profile_update', array('DispletRetsIdxUsers', 'save_from_user_page'));
add_action('displetretsidx_new_user_cron_jobs', array('DispletRetsIdxUsers', 'new_user_cron_jobs'));

?>